from subprocess import CompletedProcess
from .base import ModuleTestBase


class Testwpscan(ModuleTestBase):
    targets = ["http://127.0.0.1:8888"]
    modules_overrides = ["httpx", "wpscan"]

    wpscan_output_json = """{
  "banner": {
    "description": "WordPress Security Scanner by the WPScan Team",
    "version": "3.8.25",
    "authors": [
      "@_WPScan_",
      "@ethicalhack3r",
      "@erwan_lr",
      "@firefart"
    ],
    "sponsor": "Sponsored by Automattic - https://automattic.com/"
  },
  "start_time": 1717183319,
  "start_memory": 49950720,
  "target_url": "http://127.0.0.1:8888/",
  "target_ip": "172.29.64.1",
  "effective_url": "http://127.0.0.1:8888/",
  "interesting_findings": [
    {
      "url": "http://127.0.0.1:8888/",
      "to_s": "Headers",
      "type": "headers",
      "found_by": "Headers (Passive Detection)",
      "confidence": 100,
      "confirmed_by": {

      },
      "references": {

      },
      "interesting_entries": [
        "Server: Apache/2.4.38 (Debian)",
        "X-Powered-By: PHP/7.1.33"
      ]
    },
    {
      "url": "http://127.0.0.1:8888/xmlrpc.php",
      "to_s": "XML-RPC seems to be enabled: http://127.0.0.1:8888/xmlrpc.php",
      "type": "xmlrpc",
      "found_by": "Direct Access (Aggressive Detection)",
      "confidence": 100,
      "confirmed_by": {

      },
      "references": {
        "url": [
          "http://codex.wordpress.org/XML-RPC_Pingback_API"
        ],
        "metasploit": [
          "auxiliary/scanner/http/wordpress_ghost_scanner",
          "auxiliary/dos/http/wordpress_xmlrpc_dos",
          "auxiliary/scanner/http/wordpress_xmlrpc_login",
          "auxiliary/scanner/http/wordpress_pingback_access"
        ]
      },
      "interesting_entries": [

      ]
    },
    {
      "url": "http://127.0.0.1:8888/readme.html",
      "to_s": "WordPress readme found: http://127.0.0.1:8888/readme.html",
      "type": "readme",
      "found_by": "Direct Access (Aggressive Detection)",
      "confidence": 100,
      "confirmed_by": {

      },
      "references": {

      },
      "interesting_entries": [
        "/wp-admin/",
        "/wp-admin/admin-ajax.php",
        " "
      ]
    },
    {
      "url": "http://127.0.0.1:8888/wp-cron.php",
      "to_s": "The external WP-Cron seems to be enabled: http://127.0.0.1:8888/wp-cron.php",
      "type": "wp_cron",
      "found_by": "Direct Access (Aggressive Detection)",
      "confidence": 60,
      "confirmed_by": {

      },
      "references": {
        "url": [
          "https://www.iplocation.net/defend-wordpress-from-ddos",
          "https://github.com/wpscanteam/wpscan/issues/1299"
        ]
      },
      "interesting_entries": [

      ]
    }
  ],
  "version": {
    "number": "5.3",
    "release_date": "2019-11-12",
    "status": "insecure",
    "found_by": "Emoji Settings (Passive Detection)",
    "confidence": 100,
    "interesting_entries": [
      "http://127.0.0.1:8888/, Match: 'wp-includes\\/js\\/wp-emoji-release.min.js?ver=5.3'"
    ],
    "confirmed_by": {
      "Meta Generator (Passive Detection)": {
        "confidence": 60,
        "interesting_entries": [
          "http://127.0.0.1:8888/, Match: 'WordPress 5.3'"
        ]
      }
    },
    "vulnerabilities": [
      {
        "title": "WordPress <= 5.3 - Authenticated Improper Access Controls in REST API",
        "fixed_in": "5.3.1",
        "references": {
          "cve": [
            "2019-20043",
            "2019-16788"
          ],
          "url": [
            "https://wordpress.org/news/2019/12/wordpress-5-3-1-security-and-maintenance-release/",
            "https://github.com/WordPress/wordpress-develop/security/advisories/GHSA-g7rg-hchx-c2gw"
          ],
          "wpvulndb": [
            "4a6de154-5fbd-4c80-acd3-8902ee431bd8"
          ]
        }
      },
      {
        "title": "WordPress <= 5.3 - Authenticated Stored XSS via Crafted Links",
        "fixed_in": "5.3.1",
        "references": {
          "cve": [
            "2019-20042"
          ],
          "url": [
            "https://wordpress.org/news/2019/12/wordpress-5-3-1-security-and-maintenance-release/",
            "https://hackerone.com/reports/509930",
            "https://github.com/WordPress/wordpress-develop/commit/1f7f3f1f59567e2504f0fbebd51ccf004b3ccb1d",
            "https://github.com/WordPress/wordpress-develop/security/advisories/GHSA-xvg2-m2f4-83m7"
          ],
          "wpvulndb": [
            "23553517-34e3-40a9-a406-f3ffbe9dd265"
          ]
        }
      },
      {
        "title": "WordPress <= 5.3 - Authenticated Stored XSS via Block Editor Content",
        "fixed_in": "5.3.1",
        "references": {
          "cve": [
            "2019-16781",
            "2019-16780"
          ],
          "url": [
            "https://wordpress.org/news/2019/12/wordpress-5-3-1-security-and-maintenance-release/",
            "https://github.com/WordPress/wordpress-develop/security/advisories/GHSA-pg4x-64rh-3c9v"
          ],
          "wpvulndb": [
            "be794159-4486-4ae1-a5cc-5c190e5ddf5f"
          ]
        }
      },
      {
        "title": "WordPress <= 5.3 - wp_kses_bad_protocol() Colon Bypass",
        "fixed_in": "5.3.1",
        "references": {
          "cve": [
            "2019-20041"
          ],
          "url": [
            "https://wordpress.org/news/2019/12/wordpress-5-3-1-security-and-maintenance-release/",
            "https://github.com/WordPress/wordpress-develop/commit/b1975463dd995da19bb40d3fa0786498717e3c53"
          ],
          "wpvulndb": [
            "8fac612b-95d2-477a-a7d6-e5ec0bb9ca52"
          ]
        }
      },
      {
        "title": "WordPress < 5.4.1 - Password Reset Tokens Failed to Be Properly Invalidated",
        "fixed_in": "5.3.3",
        "references": {
          "cve": [
            "2020-11027"
          ],
          "url": [
            "https://wordpress.org/news/2020/04/wordpress-5-4-1/",
            "https://core.trac.wordpress.org/changeset/47634/",
            "https://www.wordfence.com/blog/2020/04/unpacking-the-7-vulnerabilities-fixed-in-todays-wordpress-5-4-1-security-update/",
            "https://github.com/WordPress/wordpress-develop/security/advisories/GHSA-ww7v-jg8c-q6jw"
          ],
          "wpvulndb": [
            "7db191c0-d112-4f08-a419-a1cd81928c4e"
          ]
        }
      },
      {
        "title": "WordPress < 5.4.1 - Unauthenticated Users View Private Posts",
        "fixed_in": "5.3.3",
        "references": {
          "cve": [
            "2020-11028"
          ],
          "url": [
            "https://wordpress.org/news/2020/04/wordpress-5-4-1/",
            "https://core.trac.wordpress.org/changeset/47635/",
            "https://www.wordfence.com/blog/2020/04/unpacking-the-7-vulnerabilities-fixed-in-todays-wordpress-5-4-1-security-update/",
            "https://github.com/WordPress/wordpress-develop/security/advisories/GHSA-xhx9-759f-6p2w"
          ],
          "wpvulndb": [
            "d1e1ba25-98c9-4ae7-8027-9632fb825a56"
          ]
        }
      },
      {
        "title": "WordPress < 5.4.1 - Authenticated Cross-Site Scripting (XSS) in Customizer",
        "fixed_in": "5.3.3",
        "references": {
          "cve": [
            "2020-11025"
          ],
          "url": [
            "https://wordpress.org/news/2020/04/wordpress-5-4-1/",
            "https://core.trac.wordpress.org/changeset/47633/",
            "https://www.wordfence.com/blog/2020/04/unpacking-the-7-vulnerabilities-fixed-in-todays-wordpress-5-4-1-security-update/",
            "https://github.com/WordPress/wordpress-develop/security/advisories/GHSA-4mhg-j6fx-5g3c"
          ],
          "wpvulndb": [
            "4eee26bd-a27e-4509-a3a5-8019dd48e429"
          ]
        }
      },
      {
        "title": "WordPress < 5.4.1 - Authenticated Cross-Site Scripting (XSS) in Search Block",
        "fixed_in": "5.3.3",
        "references": {
          "cve": [
            "2020-11030"
          ],
          "url": [
            "https://wordpress.org/news/2020/04/wordpress-5-4-1/",
            "https://core.trac.wordpress.org/changeset/47636/",
            "https://www.wordfence.com/blog/2020/04/unpacking-the-7-vulnerabilities-fixed-in-todays-wordpress-5-4-1-security-update/",
            "https://github.com/WordPress/wordpress-develop/security/advisories/GHSA-vccm-6gmc-qhjh"
          ],
          "wpvulndb": [
            "e4bda91b-067d-45e4-a8be-672ccf8b1a06"
          ]
        }
      },
      {
        "title": "WordPress < 5.4.1 - Cross-Site Scripting (XSS) in wp-object-cache",
        "fixed_in": "5.3.3",
        "references": {
          "cve": [
            "2020-11029"
          ],
          "url": [
            "https://wordpress.org/news/2020/04/wordpress-5-4-1/",
            "https://core.trac.wordpress.org/changeset/47637/",
            "https://www.wordfence.com/blog/2020/04/unpacking-the-7-vulnerabilities-fixed-in-todays-wordpress-5-4-1-security-update/",
            "https://github.com/WordPress/wordpress-develop/security/advisories/GHSA-568w-8m88-8g2c"
          ],
          "wpvulndb": [
            "e721d8b9-a38f-44ac-8520-b4a9ed6a5157"
          ]
        }
      },
      {
        "title": "WordPress < 5.4.1 - Authenticated Cross-Site Scripting (XSS) in File Uploads",
        "fixed_in": "5.3.3",
        "references": {
          "cve": [
            "2020-11026"
          ],
          "url": [
            "https://wordpress.org/news/2020/04/wordpress-5-4-1/",
            "https://core.trac.wordpress.org/changeset/47638/",
            "https://www.wordfence.com/blog/2020/04/unpacking-the-7-vulnerabilities-fixed-in-todays-wordpress-5-4-1-security-update/",
            "https://github.com/WordPress/wordpress-develop/security/advisories/GHSA-3gw2-4656-pfr2",
            "https://hackerone.com/reports/179695"
          ],
          "wpvulndb": [
            "55438b63-5fc9-4812-afc4-2f1eff800d5f"
          ]
        }
      },
      {
        "title": "WordPress < 5.4.2 - Authenticated XSS in Block Editor",
        "fixed_in": "5.3.4",
        "references": {
          "cve": [
            "2020-4046"
          ],
          "url": [
            "https://wordpress.org/news/2020/06/wordpress-5-4-2-security-and-maintenance-release/",
            "https://github.com/WordPress/wordpress-develop/security/advisories/GHSA-rpwf-hrh2-39jf",
            "https://pentest.co.uk/labs/research/subtle-stored-xss-wordpress-core/"
          ],
          "youtube": [
            "https://www.youtube.com/watch?v=tCh7Y8z8fb4"
          ],
          "wpvulndb": [
            "831e4a94-239c-4061-b66e-f5ca0dbb84fa"
          ]
        }
      },
      {
        "title": "WordPress < 5.4.2 - Authenticated XSS via Media Files",
        "fixed_in": "5.3.4",
        "references": {
          "cve": [
            "2020-4047"
          ],
          "url": [
            "https://wordpress.org/news/2020/06/wordpress-5-4-2-security-and-maintenance-release/",
            "https://github.com/WordPress/wordpress-develop/security/advisories/GHSA-8q2w-5m27-wm27"
          ],
          "wpvulndb": [
            "741d07d1-2476-430a-b82f-e1228a9343a4"
          ]
        }
      },
      {
        "title": "WordPress < 5.4.2 - Open Redirection",
        "fixed_in": "5.3.4",
        "references": {
          "cve": [
            "2020-4048"
          ],
          "url": [
            "https://wordpress.org/news/2020/06/wordpress-5-4-2-security-and-maintenance-release/",
            "https://github.com/WordPress/WordPress/commit/10e2a50c523cf0b9785555a688d7d36a40fbeccf",
            "https://github.com/WordPress/wordpress-develop/security/advisories/GHSA-q6pw-gvf4-5fj5"
          ],
          "wpvulndb": [
            "12855f02-432e-4484-af09-7d0fbf596909"
          ]
        }
      },
      {
        "title": "WordPress < 5.4.2 - Authenticated Stored XSS via Theme Upload",
        "fixed_in": "5.3.4",
        "references": {
          "cve": [
            "2020-4049"
          ],
          "exploitdb": [
            "48770"
          ],
          "url": [
            "https://wordpress.org/news/2020/06/wordpress-5-4-2-security-and-maintenance-release/",
            "https://github.com/WordPress/wordpress-develop/security/advisories/GHSA-87h4-phjv-rm6p",
            "https://hackerone.com/reports/406289"
          ],
          "wpvulndb": [
            "d8addb42-e70b-4439-b828-fd0697e5d9d4"
          ]
        }
      },
      {
        "title": "WordPress < 5.4.2 - Misuse of set-screen-option Leading to Privilege Escalation",
        "fixed_in": "5.3.4",
        "references": {
          "cve": [
            "2020-4050"
          ],
          "url": [
            "https://wordpress.org/news/2020/06/wordpress-5-4-2-security-and-maintenance-release/",
            "https://github.com/WordPress/WordPress/commit/dda0ccdd18f6532481406cabede19ae2ed1f575d",
            "https://github.com/WordPress/wordpress-develop/security/advisories/GHSA-4vpv-fgg2-gcqc"
          ],
          "wpvulndb": [
            "b6f69ff1-4c11-48d2-b512-c65168988c45"
          ]
        }
      },
      {
        "title": "WordPress < 5.4.2 - Disclosure of Password-Protected Page/Post Comments",
        "fixed_in": "5.3.4",
        "references": {
          "cve": [
            "2020-25286"
          ],
          "url": [
            "https://wordpress.org/news/2020/06/wordpress-5-4-2-security-and-maintenance-release/",
            "https://github.com/WordPress/WordPress/commit/c075eec24f2f3214ab0d0fb0120a23082e6b1122"
          ],
          "wpvulndb": [
            "eea6dbf5-e298-44a7-9b0d-f078ad4741f9"
          ]
        }
      },
      {
        "title": "WordPress 4.7-5.7 - Authenticated Password Protected Pages Exposure",
        "fixed_in": "5.3.7",
        "references": {
          "cve": [
            "2021-29450"
          ],
          "url": [
            "https://wordpress.org/news/2021/04/wordpress-5-7-1-security-and-maintenance-release/",
            "https://blog.wpscan.com/2021/04/15/wordpress-571-security-vulnerability-release.html",
            "https://github.com/WordPress/wordpress-develop/security/advisories/GHSA-pmmh-2f36-wvhq",
            "https://core.trac.wordpress.org/changeset/50717/"
          ],
          "youtube": [
            "https://www.youtube.com/watch?v=J2GXmxAdNWs"
          ],
          "wpvulndb": [
            "6a3ec618-c79e-4b9c-9020-86b157458ac5"
          ]
        }
      },
      {
        "title": "WordPress 3.7 to 5.7.1 - Object Injection in PHPMailer",
        "fixed_in": "5.3.8",
        "references": {
          "cve": [
            "2020-36326",
            "2018-19296"
          ],
          "url": [
            "https://github.com/WordPress/WordPress/commit/267061c9595fedd321582d14c21ec9e7da2dcf62",
            "https://wordpress.org/news/2021/05/wordpress-5-7-2-security-release/",
            "https://github.com/PHPMailer/PHPMailer/commit/e2e07a355ee8ff36aba21d0242c5950c56e4c6f9",
            "https://www.wordfence.com/blog/2021/05/wordpress-5-7-2-security-release-what-you-need-to-know/"
          ],
          "youtube": [
            "https://www.youtube.com/watch?v=HaW15aMzBUM"
          ],
          "wpvulndb": [
            "4cd46653-4470-40ff-8aac-318bee2f998d"
          ]
        }
      },
      {
        "title": "WordPress < 5.8.2 - Expired DST Root CA X3 Certificate",
        "fixed_in": "5.3.10",
        "references": {
          "url": [
            "https://wordpress.org/news/2021/11/wordpress-5-8-2-security-and-maintenance-release/",
            "https://core.trac.wordpress.org/ticket/54207"
          ],
          "wpvulndb": [
            "cc23344a-5c91-414a-91e3-c46db614da8d"
          ]
        }
      },
      {
        "title": "WordPress < 5.8 - Plugin Confusion",
        "fixed_in": "5.8",
        "references": {
          "cve": [
            "2021-44223"
          ],
          "url": [
            "https://vavkamil.cz/2021/11/25/wordpress-plugin-confusion-update-can-get-you-pwned/"
          ],
          "wpvulndb": [
            "95e01006-84e4-4e95-b5d7-68ea7b5aa1a8"
          ]
        }
      },
      {
        "title": "WordPress < 5.8.3 - SQL Injection via WP_Query",
        "fixed_in": "5.3.11",
        "references": {
          "cve": [
            "2022-21661"
          ],
          "url": [
            "https://github.com/WordPress/wordpress-develop/security/advisories/GHSA-6676-cqfm-gw84",
            "https://hackerone.com/reports/1378209"
          ],
          "wpvulndb": [
            "7f768bcf-ed33-4b22-b432-d1e7f95c1317"
          ]
        }
      },
      {
        "title": "WordPress < 5.8.3 - Author+ Stored XSS via Post Slugs",
        "fixed_in": "5.3.11",
        "references": {
          "cve": [
            "2022-21662"
          ],
          "url": [
            "https://github.com/WordPress/wordpress-develop/security/advisories/GHSA-699q-3hj9-889w",
            "https://hackerone.com/reports/425342",
            "https://blog.sonarsource.com/wordpress-stored-xss-vulnerability"
          ],
          "wpvulndb": [
            "dc6f04c2-7bf2-4a07-92b5-dd197e4d94c8"
          ]
        }
      },
      {
        "title": "WordPress 4.1-5.8.2 - SQL Injection via WP_Meta_Query",
        "fixed_in": "5.3.11",
        "references": {
          "cve": [
            "2022-21664"
          ],
          "url": [
            "https://github.com/WordPress/wordpress-develop/security/advisories/GHSA-jp3p-gw8h-6x86"
          ],
          "wpvulndb": [
            "24462ac4-7959-4575-97aa-a6dcceeae722"
          ]
        }
      },
      {
        "title": "WordPress < 5.8.3 - Super Admin Object Injection in Multisites",
        "fixed_in": "5.3.11",
        "references": {
          "cve": [
            "2022-21663"
          ],
          "url": [
            "https://github.com/WordPress/wordpress-develop/security/advisories/GHSA-jmmq-m8p8-332h",
            "https://hackerone.com/reports/541469"
          ],
          "wpvulndb": [
            "008c21ab-3d7e-4d97-b6c3-db9d83f390a7"
          ]
        }
      },
      {
        "title": "WordPress < 5.9.2 - Prototype Pollution in jQuery",
        "fixed_in": "5.3.12",
        "references": {
          "url": [
            "https://wordpress.org/news/2022/03/wordpress-5-9-2-security-maintenance-release/"
          ],
          "wpvulndb": [
            "1ac912c1-5e29-41ac-8f76-a062de254c09"
          ]
        }
      },
      {
        "title": "WP < 6.0.2 - Reflected Cross-Site Scripting",
        "fixed_in": "5.3.13",
        "references": {
          "url": [
            "https://wordpress.org/news/2022/08/wordpress-6-0-2-security-and-maintenance-release/"
          ],
          "wpvulndb": [
            "622893b0-c2c4-4ee7-9fa1-4cecef6e36be"
          ]
        }
      },
      {
        "title": "WP < 6.0.2 - Authenticated Stored Cross-Site Scripting",
        "fixed_in": "5.3.13",
        "references": {
          "url": [
            "https://wordpress.org/news/2022/08/wordpress-6-0-2-security-and-maintenance-release/"
          ],
          "wpvulndb": [
            "3b1573d4-06b4-442b-bad5-872753118ee0"
          ]
        }
      },
      {
        "title": "WP < 6.0.2 - SQLi via Link API",
        "fixed_in": "5.3.13",
        "references": {
          "url": [
            "https://wordpress.org/news/2022/08/wordpress-6-0-2-security-and-maintenance-release/"
          ],
          "wpvulndb": [
            "601b0bf9-fed2-4675-aec7-fed3156a022f"
          ]
        }
      },
      {
        "title": "WP < 6.0.3 - Stored XSS via wp-mail.php",
        "fixed_in": "5.3.14",
        "references": {
          "url": [
            "https://wordpress.org/news/2022/10/wordpress-6-0-3-security-release/",
            "https://github.com/WordPress/wordpress-develop/commit/abf236fdaf94455e7bc6e30980cf70401003e283"
          ],
          "wpvulndb": [
            "713bdc8b-ab7c-46d7-9847-305344a579c4"
          ]
        }
      },
      {
        "title": "WP < 6.0.3 - Open Redirect via wp_nonce_ays",
        "fixed_in": "5.3.14",
        "references": {
          "url": [
            "https://wordpress.org/news/2022/10/wordpress-6-0-3-security-release/",
            "https://github.com/WordPress/wordpress-develop/commit/506eee125953deb658307bb3005417cb83f32095"
          ],
          "wpvulndb": [
            "926cd097-b36f-4d26-9c51-0dfab11c301b"
          ]
        }
      },
      {
        "title": "WP < 6.0.3 - Email Address Disclosure via wp-mail.php",
        "fixed_in": "5.3.14",
        "references": {
          "url": [
            "https://wordpress.org/news/2022/10/wordpress-6-0-3-security-release/",
            "https://github.com/WordPress/wordpress-develop/commit/5fcdee1b4d72f1150b7b762ef5fb39ab288c8d44"
          ],
          "wpvulndb": [
            "c5675b59-4b1d-4f64-9876-068e05145431"
          ]
        }
      },
      {
        "title": "WP < 6.0.3 - Reflected XSS via SQLi in Media Library",
        "fixed_in": "5.3.14",
        "references": {
          "url": [
            "https://wordpress.org/news/2022/10/wordpress-6-0-3-security-release/",
            "https://github.com/WordPress/wordpress-develop/commit/8836d4682264e8030067e07f2f953a0f66cb76cc"
          ],
          "wpvulndb": [
            "cfd8b50d-16aa-4319-9c2d-b227365c2156"
          ]
        }
      },
      {
        "title": "WP < 6.0.3 - CSRF in wp-trackback.php",
        "fixed_in": "5.3.14",
        "references": {
          "url": [
            "https://wordpress.org/news/2022/10/wordpress-6-0-3-security-release/",
            "https://github.com/WordPress/wordpress-develop/commit/a4f9ca17fae0b7d97ff807a3c234cf219810fae0"
          ],
          "wpvulndb": [
            "b60a6557-ae78-465c-95bc-a78cf74a6dd0"
          ]
        }
      },
      {
        "title": "WP < 6.0.3 - Stored XSS via the Customizer",
        "fixed_in": "5.3.14",
        "references": {
          "url": [
            "https://wordpress.org/news/2022/10/wordpress-6-0-3-security-release/",
            "https://github.com/WordPress/wordpress-develop/commit/2ca28e49fc489a9bb3c9c9c0d8907a033fe056ef"
          ],
          "wpvulndb": [
            "2787684c-aaef-4171-95b4-ee5048c74218"
          ]
        }
      },
      {
        "title": "WP < 6.0.3 - Stored XSS via Comment Editing",
        "fixed_in": "5.3.14",
        "references": {
          "url": [
            "https://wordpress.org/news/2022/10/wordpress-6-0-3-security-release/",
            "https://github.com/WordPress/wordpress-develop/commit/89c8f7919460c31c0f259453b4ffb63fde9fa955"
          ],
          "wpvulndb": [
            "02d76d8e-9558-41a5-bdb6-3957dc31563b"
          ]
        }
      },
      {
        "title": "WP < 6.0.3 - Content from Multipart Emails Leaked",
        "fixed_in": "5.3.14",
        "references": {
          "url": [
            "https://wordpress.org/news/2022/10/wordpress-6-0-3-security-release/",
            "https://github.com/WordPress/wordpress-develop/commit/3765886b4903b319764490d4ad5905bc5c310ef8"
          ],
          "wpvulndb": [
            "3f707e05-25f0-4566-88ed-d8d0aff3a872"
          ]
        }
      },
      {
        "title": "WP < 6.0.3 - SQLi in WP_Date_Query",
        "fixed_in": "5.3.14",
        "references": {
          "url": [
            "https://wordpress.org/news/2022/10/wordpress-6-0-3-security-release/",
            "https://github.com/WordPress/wordpress-develop/commit/d815d2e8b2a7c2be6694b49276ba3eee5166c21f"
          ],
          "wpvulndb": [
            "1da03338-557f-4cb6-9a65-3379df4cce47"
          ]
        }
      },
      {
        "title": "WP < 6.0.3 - Stored XSS via RSS Widget",
        "fixed_in": "5.3.14",
        "references": {
          "url": [
            "https://wordpress.org/news/2022/10/wordpress-6-0-3-security-release/",
            "https://github.com/WordPress/wordpress-develop/commit/929cf3cb9580636f1ae3fe944b8faf8cca420492"
          ],
          "wpvulndb": [
            "58d131f5-f376-4679-b604-2b888de71c5b"
          ]
        }
      },
      {
        "title": "WP < 6.0.3 - Data Exposure via REST Terms/Tags Endpoint",
        "fixed_in": "5.3.14",
        "references": {
          "url": [
            "https://wordpress.org/news/2022/10/wordpress-6-0-3-security-release/",
            "https://github.com/WordPress/wordpress-develop/commit/ebaac57a9ac0174485c65de3d32ea56de2330d8e"
          ],
          "wpvulndb": [
            "b27a8711-a0c0-4996-bd6a-01734702913e"
          ]
        }
      },
      {
        "title": "WP < 6.0.3 - Multiple Stored XSS via Gutenberg",
        "fixed_in": "5.3.14",
        "references": {
          "url": [
            "https://wordpress.org/news/2022/10/wordpress-6-0-3-security-release/",
            "https://github.com/WordPress/gutenberg/pull/45045/files"
          ],
          "wpvulndb": [
            "f513c8f6-2e1c-45ae-8a58-36b6518e2aa9"
          ]
        }
      },
      {
        "title": "WP <= 6.2 - Unauthenticated Blind SSRF via DNS Rebinding",
        "fixed_in": null,
        "references": {
          "cve": [
            "2022-3590"
          ],
          "url": [
            "https://blog.sonarsource.com/wordpress-core-unauthenticated-blind-ssrf/"
          ],
          "wpvulndb": [
            "c8814e6e-78b3-4f63-a1d3-6906a84c1f11"
          ]
        }
      },
      {
        "title": "WP < 6.2.1 - Directory Traversal via Translation Files",
        "fixed_in": "5.3.15",
        "references": {
          "cve": [
            "2023-2745"
          ],
          "url": [
            "https://wordpress.org/news/2023/05/wordpress-6-2-1-maintenance-security-release/"
          ],
          "wpvulndb": [
            "2999613a-b8c8-4ec0-9164-5dfe63adf6e6"
          ]
        }
      },
      {
        "title": "WP < 6.2.1 - Thumbnail Image Update via CSRF",
        "fixed_in": "5.3.15",
        "references": {
          "url": [
            "https://wordpress.org/news/2023/05/wordpress-6-2-1-maintenance-security-release/"
          ],
          "wpvulndb": [
            "a03d744a-9839-4167-a356-3e7da0f1d532"
          ]
        }
      },
      {
        "title": "WP < 6.2.1 - Contributor+ Stored XSS via Open Embed Auto Discovery",
        "fixed_in": "5.3.15",
        "references": {
          "url": [
            "https://wordpress.org/news/2023/05/wordpress-6-2-1-maintenance-security-release/"
          ],
          "wpvulndb": [
            "3b574451-2852-4789-bc19-d5cc39948db5"
          ]
        }
      },
      {
        "title": "WP < 6.2.2 - Shortcode Execution in User Generated Data",
        "fixed_in": "5.3.15",
        "references": {
          "url": [
            "https://wordpress.org/news/2023/05/wordpress-6-2-1-maintenance-security-release/",
            "https://wordpress.org/news/2023/05/wordpress-6-2-2-security-release/"
          ],
          "wpvulndb": [
            "ef289d46-ea83-4fa5-b003-0352c690fd89"
          ]
        }
      },
      {
        "title": "WP < 6.2.1 - Contributor+ Content Injection",
        "fixed_in": "5.3.15",
        "references": {
          "url": [
            "https://wordpress.org/news/2023/05/wordpress-6-2-1-maintenance-security-release/"
          ],
          "wpvulndb": [
            "1527ebdb-18bc-4f9d-9c20-8d729a628670"
          ]
        }
      },
      {
        "title": "WP < 6.3.2 - Denial of Service via Cache Poisoning",
        "fixed_in": "5.3.16",
        "references": {
          "url": [
            "https://wordpress.org/news/2023/10/wordpress-6-3-2-maintenance-and-security-release/"
          ],
          "wpvulndb": [
            "6d80e09d-34d5-4fda-81cb-e703d0e56e4f"
          ]
        }
      },
      {
        "title": "WP < 6.3.2 - Subscriber+ Arbitrary Shortcode Execution",
        "fixed_in": "5.3.16",
        "references": {
          "url": [
            "https://wordpress.org/news/2023/10/wordpress-6-3-2-maintenance-and-security-release/"
          ],
          "wpvulndb": [
            "3615aea0-90aa-4f9a-9792-078a90af7f59"
          ]
        }
      },
      {
        "title": "WP < 6.3.2 - Contributor+ Comment Disclosure",
        "fixed_in": "5.3.16",
        "references": {
          "cve": [
            "2023-39999"
          ],
          "url": [
            "https://wordpress.org/news/2023/10/wordpress-6-3-2-maintenance-and-security-release/"
          ],
          "wpvulndb": [
            "d35b2a3d-9b41-4b4f-8e87-1b8ccb370b9f"
          ]
        }
      },
      {
        "title": "WP < 6.3.2 - Unauthenticated Post Author Email Disclosure",
        "fixed_in": "5.3.16",
        "references": {
          "cve": [
            "2023-5561"
          ],
          "url": [
            "https://wpscan.com/blog/email-leak-oracle-vulnerability-addressed-in-wordpress-6-3-2/",
            "https://wordpress.org/news/2023/10/wordpress-6-3-2-maintenance-and-security-release/"
          ],
          "wpvulndb": [
            "19380917-4c27-4095-abf1-eba6f913b441"
          ]
        }
      },
      {
        "title": "WordPress < 6.4.3 - Deserialization of Untrusted Data",
        "fixed_in": "5.3.17",
        "references": {
          "url": [
            "https://wordpress.org/news/2024/01/wordpress-6-4-3-maintenance-and-security-release/"
          ],
          "wpvulndb": [
            "5e9804e5-bbd4-4836-a5f0-b4388cc39225"
          ]
        }
      },
      {
        "title": "WordPress < 6.4.3 - Admin+ PHP File Upload",
        "fixed_in": "5.3.17",
        "references": {
          "url": [
            "https://wordpress.org/news/2024/01/wordpress-6-4-3-maintenance-and-security-release/"
          ],
          "wpvulndb": [
            "a8e12fbe-c70b-4078-9015-cf57a05bdd4a"
          ]
        }
      }
    ]
  },
  "main_theme": null,
  "plugins": {
    "social-warfare": {
      "slug": "social-warfare",
      "location": "http://127.0.0.1:8888/wp-content/plugins/social-warfare/",
      "latest_version": "4.4.6.3",
      "last_updated": "2024-04-07T19:32:00.000Z",
      "outdated": true,
      "readme_url": null,
      "directory_listing": null,
      "error_log_url": null,
      "found_by": "Comment (Passive Detection)",
      "confidence": 30,
      "interesting_entries": [

      ],
      "confirmed_by": {

      },
      "vulnerabilities": [
        {
          "title": "Social Warfare <= 3.5.2 - Unauthenticated Arbitrary Settings Update",
          "fixed_in": "3.5.3",
          "references": {
            "cve": [
              "2019-9978"
            ],
            "url": [
              "https://wordpress.org/support/topic/malware-into-new-update/",
              "https://www.wordfence.com/blog/2019/03/unpatched-zero-day-vulnerability-in-social-warfare-plugin-exploited-in-the-wild/",
              "https://threatpost.com/wordpress-plugin-removed-after-zero-day-discovered/143051/",
              "https://twitter.com/warfareplugins/status/1108826025188909057",
              "https://www.wordfence.com/blog/2019/03/recent-social-warfare-vulnerability-allowed-remote-code-execution/"
            ],
            "wpvulndb": [
              "32085d2d-1235-42b4-baeb-bc43172a4972"
            ]
          }
        },
        {
          "title": "Social Warfare <= 3.5.2 - Unauthenticated Remote Code Execution (RCE)",
          "fixed_in": "3.5.3",
          "references": {
            "url": [
              "https://www.webarxsecurity.com/social-warfare-vulnerability/"
            ],
            "wpvulndb": [
              "7b412469-cc03-4899-b397-38580ced5618"
            ]
          }
        },
        {
          "title": "Social Warfare < 4.3.1 - Subscriber+ Post Meta Deletion",
          "fixed_in": "4.3.1",
          "references": {
            "cve": [
              "2023-0402"
            ],
            "wpvulndb": [
              "5116068f-4b84-42ad-a88d-03e46096b41c"
            ]
          }
        },
        {
          "title": "Social Warfare < 4.4.0 - Post Meta Deletion via CSRF",
          "fixed_in": "4.4.0",
          "references": {
            "cve": [
              "2023-0403"
            ],
            "wpvulndb": [
              "7140abf5-5966-4361-bd51-ee29d3071a30"
            ]
          }
        },
        {
          "title": "Social Sharing Plugin - Social Warfare < 4.4.4 - Authenticated (Contributor+) Stored Cross-Site Scripting via Shortcode",
          "fixed_in": "4.4.4",
          "references": {
            "cve": [
              "2023-4842"
            ],
            "url": [
              "https://www.wordfence.com/threat-intel/vulnerabilities/id/8f5b9aff-0833-4887-ae59-df5bc88c7f91"
            ],
            "wpvulndb": [
              "ab221b58-369e-4010-ae36-be099b2f4c9b"
            ]
          }
        },
        {
          "title": "Social Sharing Plugin – Social Warfare < 4.4.6.2 - Authenticated(Contributor+) Stored Cross-Site Scripting via Shortcode",
          "fixed_in": "4.4.6.2",
          "references": {
            "cve": [
              "2024-1959"
            ],
            "url": [
              "https://www.wordfence.com/threat-intel/vulnerabilities/id/1016f16c-0ab2-4cac-a7a5-8d93a37e7894"
            ],
            "wpvulndb": [
              "26ad138e-990a-4401-84e4-ea694ccf6e7f"
            ]
          }
        },
        {
          "title": "Social Sharing Plugin – Social Warfare < 4.4.6 - Cross-Site Request Forgery",
          "fixed_in": "4.4.6",
          "references": {
            "cve": [
              "2024-34825"
            ],
            "url": [
              "https://www.wordfence.com/threat-intel/vulnerabilities/id/f105bee6-21b2-4014-bb0a-9e53c49e29b0"
            ],
            "wpvulndb": [
              "acb8b33c-6b74-4d65-a3a5-5cad0c1ea8b0"
            ]
          }
        }
      ],
      "version": {
        "number": "3.5.2",
        "confidence": 100,
        "found_by": "Comment (Passive Detection)",
        "interesting_entries": [
          "http://127.0.0.1:8888/, Match: 'Social Warfare v3.5.2'"
        ],
        "confirmed_by": {
          "Readme - Stable Tag (Aggressive Detection)": {
            "confidence": 80,
            "interesting_entries": [
              "http://127.0.0.1:8888/wp-content/plugins/social-warfare/readme.txt"
            ]
          },
          "Readme - ChangeLog Section (Aggressive Detection)": {
            "confidence": 50,
            "interesting_entries": [
              "http://127.0.0.1:8888/wp-content/plugins/social-warfare/readme.txt"
            ]
          }
        }
      }
    }
  },
  "config_backups": {

  },
  "vuln_api": {
    "plan": "free",
    "requests_done_during_scan": 0,
    "requests_remaining": 15
  },
  "stop_time": 1717183322,
  "elapsed": 3,
  "requests_done": 169,
  "cached_requests": 6,
  "data_sent": 59178,
  "data_sent_humanised": "57.791 KB",
  "data_received": 313184,
  "data_received_humanised": "305.844 KB",
  "used_memory": 225398784,
  "used_memory_humanised": "214.957 MB"
}"""

    async def setup_after_prep(self, module_test):
        async def wpscan_mock_run(*command, **kwargs):
            return CompletedProcess(command, 0, self.wpscan_output_json, "")

        module_test.monkeypatch.setattr(module_test.scan.helpers, "run", wpscan_mock_run)

    def check(self, module_test, events):
        findings = [e for e in events if e.type == "FINDING"]
        vulnerabilities = [e for e in events if e.type == "VULNERABILITY"]
        technologies = [e for e in events if e.type == "TECHNOLOGY"]
        assert len(findings) == 1
        assert len(vulnerabilities) == 59
        assert len(technologies) == 4



from .base import ModuleTestBase


class TestGraphQLIntrospectionNon200(ModuleTestBase):
    targets = ["http://127.0.0.1:8888"]
    modules_overrides = ["graphql_introspection"]

    async def setup_after_prep(self, module_test):
        module_test.set_expect_requests(
            expect_args={"method": "POST", "uri": "/"},
            respond_args={"response_data": "ok"},
        )

    def check(self, module_test, events):
        assert all(e.type != "FINDING" for e in events), "should have raised 0 events"


class TestGraphQLIntrospection(ModuleTestBase):
    targets = ["http://127.0.0.1:8888"]
    modules_overrides = ["graphql_introspection"]

    async def setup_after_prep(self, module_test):
        module_test.set_expect_requests(
            expect_args={"method": "POST", "uri": "/"},
            respond_args={
                "response_data": """{"data": {"__schema": {"types": ["dummy"]}}}""",
            },
        )

    def check(self, module_test, events):
        finding = [e for e in events if e.type == "FINDING"]
        assert finding, "should have raised 1 FINDING event"
        assert finding[0].data["url"] == "http://127.0.0.1:8888/"
        assert finding[0].data["description"] == "GraphQL schema"



import re
from bbot.modules.base import BaseModule


class medusa(BaseModule):
    watched_events = ["PROTOCOL"]
    produced_events = ["VULNERABILITY"]
    flags = ["active", "aggressive", "deadly"]
    per_host_only = True
    meta = {
        "description": "Medusa SNMP bruteforcing with v1, v2c and R/W check.",
        "created_date": "2025-05-16",
        "author": "@christianfl",
    }
    scope_distance_modifier = None

    options = {
        "snmp_wordlist": "https://raw.githubusercontent.com/danielmiessler/SecLists/refs/heads/master/Discovery/SNMP/common-snmp-community-strings.txt",
        "snmp_versions": ["1", "2C"],  # Only 1 and 2C are available with medusa 2.3.
        "wait_microseconds": 200,
        "timeout_s": 5,
        "threads": 5,
    }

    options_desc = {
        "snmp_wordlist": "Wordlist url for SNMP community strings, newline separated (default https://raw.githubusercontent.com/danielmiessler/SecLists/refs/heads/master/Discovery/SNMP/snmp.txt)",
        "snmp_versions": "List of SNMP versions to attempt against the SNMP server (default ['1', '2C'])",
        "wait_microseconds": "Wait time after every SNMP request in microseconds (default 200)",
        "timeout_s": "Wait time for the SNMP response(s) once at the end of all attempts (default 5)",
        "threads": "Number of communities to be tested concurrently (default 5)",
    }

    deps_ansible = [
        {
            "name": "Install build dependencies",
            "package": {
                "name": [
                    "autoconf",
                    "automake",
                    "libtool",
                    "gcc",
                    "make",
                ],
                "state": "present",
            },
            "become": True,
            "ignore_errors": True,
        },
        {
            "name": "Get medusa repo",
            "git": {
                "repo": "https://github.com/jmk-foofus/medusa",
                "dest": "#{BBOT_TEMP}/medusa/gitrepo",
                "version": "2.3",  # Newest stable, 2025-05-15
            },
        },
        {
            # The git repo will be copied because during build, files and subfolders get created. That prevents the Ansible git module to cache the repo.
            "name": "Copy medusa repo",
            "copy": {
                "src": "#{BBOT_TEMP}/medusa/gitrepo/",
                "dest": "#{BBOT_TEMP}/medusa/workdir/",
            },
        },
        {
            "name": "Build medusa: autoreconf",
            "command": {
                "chdir": "#{BBOT_TEMP}/medusa/workdir",
                "cmd": "autoreconf -f -i",
            },
        },
        {
            "name": "Build medusa: configure",
            "command": {
                "chdir": "#{BBOT_TEMP}/medusa/workdir",
                "cmd": "./configure --prefix=#{BBOT_TEMP}/medusa/build",
            },
        },
        {
            "name": "Build medusa: make",
            "command": {
                "chdir": "#{BBOT_TEMP}/medusa/workdir",
                "cmd": "make",
            },
        },
        {
            "name": "Build medusa: make install",
            "command": {
                "chdir": "#{BBOT_TEMP}/medusa/workdir",
                "cmd": "make install",
                "creates": "#{BBOT_TEMP}/medusa/build/bin/medusa",
            },
        },
        {
            "name": "Install medusa",
            "copy": {
                "src": "#{BBOT_TEMP}/medusa/build/bin/medusa",
                "dest": "#{BBOT_TOOLS}/",
                "mode": "u+x,g+x,o+x",
            },
        },
    ]

    async def setup_deps(self):
        self.snmp_wordlist_path = await self.helpers.wordlist(self.config.get("snmp_wordlist"))
        return True

    async def setup(self):
        self.password_match_regex = re.compile(r"Password:\s*(\S+)")
        self.success_indicator_match_regex = re.compile(r"\[([^\]]+)\]\s*$")

        return True

    async def filter_event(self, event):
        handled_protocols = ["snmp"]  # Could be extended later

        protocol = event.data["protocol"].lower()
        if not protocol in handled_protocols:
            return False, f"service {protocol} is currently not supported. Only SNMP."

        return True

    async def handle_event(self, event):
        host = str(event.host)
        port = str(event.port)
        protocol = event.data["protocol"].lower()

        if protocol == "snmp":
            snmp_versions = self.config.get("snmp_versions")

            # Medusa must be called for each SNMP version separately after each run finished.
            for snmp_version in snmp_versions:
                command = await self.construct_command(host, port, protocol, snmp_version)

                result = await self.run_process(command)

                if result.stderr:
                    # Medusa outputs to stderr if a readonly community was found in WRITE mode
                    # That's intended behavior
                    self.info(f"Medusa stderr: {result.stderr}")

                async for message in self.parse_output(result.stdout, snmp_version):
                    vuln_event = self.create_vuln_event("CRITICAL", message, event)
                    await self.emit_event(vuln_event)

        # else: Medusa supports various protocols which could in theory be implemented later on.

    async def parse_output(self, output, protocol_version):
        for line in output.splitlines():
            # Print original Medusa output
            self.info(line)

            if "FOUND" in line:
                # Some credential was guessed
                password_match = self.password_match_regex.search(line)
                password = password_match.group(1) if password_match else None

                success_indicator_match = self.success_indicator_match_regex.search(line)
                success_indicator = success_indicator_match.group(1) if success_indicator_match else None

                # Medusa in WRITE mode shows "ERROR" if a readonly community was found. Replace with "READ"
                mode = "R/W" if success_indicator == "success" else "READ" if success_indicator == "ERROR" else "MODE?"

                message = f"VALID [SNMPV{protocol_version}] CREDENTIALS FOUND: {password} [{mode}]"

                yield message

    async def construct_command(self, host, port, protocol, protocol_version):
        # -b                Suppress startup banner
        # -v                Set verbosity level (4 = Show only errors and credentials)
        # -R                Number of attempted retries
        # -M                Medusa module to execute (SNMP)
        # -T                Number of concurrent hosts
        # -t                Number of concurrent login attempts
        # -h                Target hostname or ip address
        # -u                Username to test (Empty for SNMP)
        # -P                Wordlist for passwords
        # -m                Module specific parameters:
        #   TIMEOUT:<number>        Sets the number of seconds to wait for the UDP responses (default: 5 sec).
        #   SEND_DELAY:<number>     Sets the number of microseconds to wait between sending queries (default: 200 usec).
        #   VERSION:<1|2C>          Set the SNMP client version.
        #   ACCESS:<READ|WRITE>     Set level of access to test for with the community string. ("WRITE" does include "READ")

        # Example command to bruteforce SNMP:
        #
        # medusa -b -v 4 -R 1 -M snmp -T 1 -t 1 -h 127.0.0.1 -u '' -P communities.txt -m VERSION:2C -m SEND_DELAY:1000000 -m ACCESS:WRITE -m TIMEOUT:10

        cmd = [
            "medusa",
            "-b",
            "-v",
            4,
            "-R",
            1,
            "-M",
            protocol,
            "-T",
            1,
            "-t",
            self.config.get("threads"),
            "-h",
            host,
            "-u",
            "''",
            "-P",
            self.snmp_wordlist_path,
            "-m",
            f"VERSION:{protocol_version}",
            "-m",
            f"SEND_DELAY:{self.config.get('wait_microseconds')}",
            "-m",
            "ACCESS:WRITE",
            "-m",
            f"TIMEOUT:{self.config.get('timeout_s')}",
        ]

        return cmd

    def create_vuln_event(self, severity, description, source_event):
        host = str(source_event.host)
        port = str(source_event.port)

        return self.make_event(
            {
                "severity": severity,
                "host": host,
                "port": port,
                "description": description,
            },
            "VULNERABILITY",
            source_event,
        )



import pytest
import random
from bbot.core.helpers.misc import closest_match


class TestClosestMatchBenchmarks:
    """
    Benchmark tests for closest_match operations.

    This function is critical for BBOT's DNS brute forcing, where it finds the best
    matching parent event among thousands of choices. Performance here directly impacts
    scan throughput and DNS mutation efficiency.
    """

    def setup_method(self):
        """Setup common test data"""
        # Set deterministic seed for consistent benchmark results
        random.seed(42)  # Fixed seed for reproducible results

        # Generate test data for benchmarks
        self.large_closest_match_choices = self._generate_large_closest_match_choices()
        self.realistic_closest_match_choices = self._generate_realistic_closest_match_choices()

    def _generate_large_closest_match_choices(self):
        """Generate large closest match dataset (stress test with many parent events)"""
        choices = []
        for i in range(10000):
            # Generate realistic domain names with more variety
            tld = random.choice(["com", "net", "org", "io", "co", "dev"])
            domain = f"subdomain{i}.example{i % 100}.{tld}"
            choices.append(domain)
        return choices

    def _generate_realistic_closest_match_choices(self):
        """Generate realistic closest match parent event choices (like actual BBOT usage)"""
        choices = []

        # Common TLDs
        tlds = ["com", "net", "org", "io", "co", "dev", "test", "local"]

        # Generate parent domains with realistic patterns
        for i in range(5000):
            # Base domain patterns
            if i % 10 == 0:
                # Simple domains
                domain = f"example{i}.{random.choice(tlds)}"
            elif i % 5 == 0:
                # Multi-level domains
                domain = f"sub{i}.example{i}.{random.choice(tlds)}"
            else:
                # Complex domains
                domain = f"level1{i}.level2{i}.example{i}.{random.choice(tlds)}"

            choices.append(domain)

        return choices

    @pytest.mark.benchmark(group="closest_match")
    def test_large_closest_match_lookup(self, benchmark):
        """Benchmark closest_match with large closest match workload (many parent events)"""

        def find_large_closest_match():
            return closest_match("subdomain5678.example50.com", self.large_closest_match_choices)

        result = benchmark.pedantic(find_large_closest_match, iterations=50, rounds=10)
        assert result is not None

    @pytest.mark.benchmark(group="closest_match")
    def test_realistic_closest_match_workload(self, benchmark):
        """Benchmark closest_match with realistic BBOT closest match parent event choices"""

        def find_realistic_closest_match():
            return closest_match("subdomain123.example5.com", self.realistic_closest_match_choices)

        result = benchmark.pedantic(find_realistic_closest_match, iterations=50, rounds=10)
        assert result is not None



from typing import List
from bbot import Scanner
from fastapi import FastAPI, Query

app = FastAPI()


@app.get("/start")
async def start(targets: List[str] = Query(...)):
    scanner = Scanner(*targets, modules=["httpx"])
    events = [e async for e in scanner.async_start()]
    return [e.json() for e in events]


@app.get("/ping")
async def ping():
    return {"status": "ok"}



from .base import BaseLightfuzz
from bbot.errors import HttpCompareError

from urllib.parse import quote


class path(BaseLightfuzz):
    """
    Detects path traversal and local file inclusion vulnerabilities

    Techniques:

    * Relative Path Traversal:
       - Tests various relative path traversal patterns (../, ./, .../, etc.)
       - Uses multiple encoding variations (URL encoding, double encoding)
       - Attempts various path validation bypass techniques

    * Absolute Path Traversal:
       - Tests absolute paths for Windows (c:\\windows\\win.ini)
       - Tests absolute paths for Unix (/etc/passwd)
       - Tests null byte injection for extension bypass (%00)

    Results are validated using multiple confirmations and WAF response filtering to eliminate false positives.
    """

    friendly_name = "Path Traversal"

    async def fuzz(self):
        cookies = self.event.data.get("assigned_cookies", {})
        probe_value = self.incoming_probe_value(populate_empty=False)
        if not probe_value:
            self.debug(
                f"Path Traversal detection requires original value, aborting [{self.event.data['type']}] [{self.event.data['name']}]"
            )
            return

        # Single dot traversal tolerance test
        path_techniques = {
            "single-dot traversal tolerance (no-encoding)": {
                "singledot_payload": f"./a/../{probe_value}",
                "doubledot_payload": f"../a/../{probe_value}",
            },
            "single-dot traversal tolerance (no-encoding, leading slash)": {
                "singledot_payload": f"/./a/../{probe_value}",
                "doubledot_payload": f"/../a/../{probe_value}",
            },
            "single-dot traversal tolerance (url-encoding)": {
                "singledot_payload": quote(f"./a/../{probe_value}".encode(), safe=""),
                "doubledot_payload": quote(f"../a/../{probe_value}".encode(), safe=""),
            },
            "single-dot traversal tolerance (url-encoding, leading slash)": {
                "singledot_payload": quote(f"/./a/../{probe_value}".encode(), safe=""),
                "doubledot_payload": quote(f"/../a/../{probe_value}".encode(), safe=""),
            },
            "single-dot traversal tolerance (non-recursive stripping)": {
                "singledot_payload": f"...//a/....//{probe_value}",
                "doubledot_payload": f"....//a/....//{probe_value}",
            },
            "single-dot traversal tolerance (non-recursive stripping, leading slash)": {
                "singledot_payload": f"/...//a/....//{probe_value}",
                "doubledot_payload": f"/....//a/....//{probe_value}",
            },
            "single-dot traversal tolerance (double url-encoding)": {
                "singledot_payload": f".%252fa%252f..%252f{probe_value}",
                "doubledot_payload": f"..%252fa%252f..%252f{probe_value}",
            },
            "single-dot traversal tolerance (double url-encoding, leading slash)": {
                "singledot_payload": f"%252f.%252fa%252f..%252f{probe_value}",
                "doubledot_payload": f"%252f..%252fa%252f..%252f{probe_value}",
            },
        }

        compiled_regex = self.lightfuzz.helpers.re.compile(r"/(?:[\w-]+/)*[\w-]+\.\w+")
        linux_path_regex = await self.lightfuzz.helpers.re.match(compiled_regex, probe_value)
        if linux_path_regex is not None:
            original_path_only = "/".join(probe_value.split("/")[:-1])
            original_filename_only = probe_value.split("/")[-1]
            # Some servers validate the start of the path, so we construct our payload with the original path and filename
            path_techniques["single-dot traversal tolerance (start of path validation)"] = {
                "singledot_payload": f"{original_path_only}/./{original_filename_only}",
                "doubledot_payload": f"{original_path_only}/../{original_filename_only}",
            }

        for path_technique, payloads in path_techniques.items():
            iterations = 5  # one failed detection is tolerated, as long as its not the first run
            confirmations = 0
            while iterations > 0:
                try:
                    http_compare = self.compare_baseline(
                        self.event.data["type"], probe_value, cookies, skip_urlencoding=True
                    )
                    singledot_probe = await self.compare_probe(
                        http_compare,
                        self.event.data["type"],
                        payloads["singledot_payload"],
                        cookies,
                        skip_urlencoding=True,
                    )
                    doubledot_probe = await self.compare_probe(
                        http_compare,
                        self.event.data["type"],
                        payloads["doubledot_payload"],
                        cookies,
                        skip_urlencoding=True,
                    )
                    # if singledot_probe[0] is true, the response is the same as the baseline. This indicates adding a single dot did not break the functionality
                    # next, if doubledot_probe[0] is false, the response is different from the baseline. This further indicates that a real path is being manipulated
                    # if doubledot_probe[3] is not None, the response is not empty.
                    # if doubledot_probe[1] is not ["header"], the response is not JUST a header change.
                    # "The requested URL was rejected" is a very common WAF error message which appears on 200 OK response, confusing detections
                    if (
                        singledot_probe[0] is True
                        and doubledot_probe[0] is False
                        and doubledot_probe[3] is not None
                        and doubledot_probe[1] != ["header"]
                        and "The requested URL was rejected" not in doubledot_probe[3].text
                    ):
                        confirmations += 1
                        self.verbose(f"Got possible Path Traversal detection: [{str(confirmations)}] Confirmations")
                        # only report if we have 3 confirmations
                        if confirmations > 3:
                            self.results.append(
                                {
                                    "type": "FINDING",
                                    "description": f"POSSIBLE Path Traversal. {self.metadata()} Detection Method: [{path_technique}]",
                                }
                            )
                            # no need to report both techniques if they both work
                            break
                except HttpCompareError as e:
                    iterations -= 1
                    self.debug(e)
                    continue

                iterations -= 1
                if confirmations == 0:
                    break

        # Absolute path test, covering Windows and Linux
        absolute_paths = {
            r"c:\\windows\\win.ini": "; for 16-bit app support",
            "/etc/passwd": "daemon:x:",
            "../../../../../etc/passwd%00.png": "daemon:x:",
        }

        for path, trigger in absolute_paths.items():
            r = await self.standard_probe(self.event.data["type"], cookies, path, skip_urlencoding=True)
            if r and trigger in r.text:
                self.results.append(
                    {
                        "type": "FINDING",
                        "description": f"POSSIBLE Path Traversal. {self.metadata()} Detection Method: [Absolute Path: {path}]",
                    }
                )



import regex as re
from contextlib import suppress

from bbot.modules.base import BaseModule


class azure_tenant(BaseModule):
    watched_events = ["DNS_NAME"]
    produced_events = ["DNS_NAME"]
    flags = ["affiliates", "subdomain-enum", "cloud-enum", "passive", "safe"]
    meta = {
        "description": "Query Azure for tenant sister domains",
        "created_date": "2024-07-04",
        "author": "@TheTechromancer",
    }

    base_url = "https://autodiscover-s.outlook.com"
    in_scope_only = True
    per_domain_only = True

    async def setup(self):
        self.processed = set()
        self.d_xml_regex = re.compile(r"<Domain>([^<>/]*)</Domain>", re.I)
        return True

    async def handle_event(self, event):
        _, query = self.helpers.split_domain(event.data)
        domains, openid_config = await self.query(query)

        tenant_id = None
        authorization_endpoint = openid_config.get("authorization_endpoint", "")
        matches = await self.helpers.re.findall(self.helpers.regexes.uuid_regex, authorization_endpoint)
        if matches:
            tenant_id = matches[0]

        tenant_names = set()
        if domains:
            self.verbose(f'Found {len(domains):,} domains under tenant for "{query}": {", ".join(sorted(domains))}')
            for domain in domains:
                if domain != query:
                    await self.emit_event(
                        domain,
                        "DNS_NAME",
                        parent=event,
                        tags=["affiliate", "azure-tenant"],
                        context=f'{{module}} queried Outlook autodiscover for "{query}" and found {{event.type}}: {{event.data}}',
                    )
                    # tenant names
                    if domain.lower().endswith(".onmicrosoft.com"):
                        tenantname = domain.split(".")[0].lower()
                        if tenantname:
                            tenant_names.add(tenantname)

            tenant_names = sorted(tenant_names)
            event_data = {"tenant-names": tenant_names, "domains": sorted(domains)}
            tenant_names_str = ",".join(tenant_names)
            if tenant_id is not None:
                event_data["tenant-id"] = tenant_id
            await self.emit_event(
                event_data,
                "AZURE_TENANT",
                parent=event,
                context=f'{{module}} queried Outlook autodiscover for "{query}" and found {{event.type}}: {tenant_names_str}',
            )

    async def query(self, domain):
        url = f"{self.base_url}/autodiscover/autodiscover.svc"
        data = f"""<?xml version="1.0" encoding="utf-8"?>
<soap:Envelope xmlns:exm="http://schemas.microsoft.com/exchange/services/2006/messages" xmlns:ext="http://schemas.microsoft.com/exchange/services/2006/types" xmlns:a="http://www.w3.org/2005/08/addressing" xmlns:soap="http://schemas.xmlsoap.org/soap/envelope/" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema">
    <soap:Header>
        <a:Action soap:mustUnderstand="1">http://schemas.microsoft.com/exchange/2010/Autodiscover/Autodiscover/GetFederationInformation</a:Action>
        <a:To soap:mustUnderstand="1">https://autodiscover-s.outlook.com/autodiscover/autodiscover.svc</a:To>
        <a:ReplyTo>
            <a:Address>http://www.w3.org/2005/08/addressing/anonymous</a:Address>
        </a:ReplyTo>
    </soap:Header>
    <soap:Body>
        <GetFederationInformationRequestMessage xmlns="http://schemas.microsoft.com/exchange/2010/Autodiscover">
            <Request>
                <Domain>{domain}</Domain>
            </Request>
        </GetFederationInformationRequestMessage>
    </soap:Body>
</soap:Envelope>"""

        headers = {
            "Content-Type": "text/xml; charset=utf-8",
            "SOAPAction": '"http://schemas.microsoft.com/exchange/2010/Autodiscover/Autodiscover/GetFederationInformation"',
            "User-Agent": "AutodiscoverClient",
            "Accept-Encoding": "identity",
        }

        self.debug(f"Retrieving tenant domains at {url}")

        autodiscover_task = self.helpers.create_task(
            self.helpers.request(url, method="POST", headers=headers, content=data)
        )
        openid_url = f"https://login.windows.net/{domain}/.well-known/openid-configuration"
        openid_task = self.helpers.create_task(self.helpers.request(openid_url))

        r = await autodiscover_task
        status_code = getattr(r, "status_code", 0)
        if status_code not in (200, 421):
            self.verbose(f'Error retrieving azure_tenant domains for "{domain}" (status code: {status_code})')
            return set(), {}
        found_domains = list(set(await self.helpers.re.findall(self.d_xml_regex, r.text)))
        domains = set()

        for d in found_domains:
            # make sure we don't make any unnecessary api calls
            d = str(d).lower()
            _, query = self.helpers.split_domain(d)
            self.processed.add(hash(query))
            domains.add(d)
            # absorb into word cloud
            self.scan.word_cloud.absorb_word(d)

        r = await openid_task
        openid_config = {}
        with suppress(Exception):
            openid_config = r.json()

        domains = sorted(domains)
        return domains, openid_config



import os
import atexit
from contextlib import suppress


class SharedInterpreterState:
    """
    A class to track the primary BBOT process.

    Used to prevent spawning multiple unwanted processes with multiprocessing.
    """

    def __init__(self):
        self.main_process_var_name = "_BBOT_MAIN_PID"
        self.scan_process_var_name = "_BBOT_SCAN_PID"
        atexit.register(self.cleanup)

    @property
    def is_main_process(self):
        is_main_process = self.main_pid == os.getpid()
        return is_main_process

    @property
    def is_scan_process(self):
        is_scan_process = os.getpid() == self.scan_pid
        return is_scan_process

    @property
    def main_pid(self):
        main_pid = int(os.environ.get(self.main_process_var_name, 0))
        if main_pid == 0:
            main_pid = os.getpid()
            # if main PID is not set, set it to the current PID
            os.environ[self.main_process_var_name] = str(main_pid)
        return main_pid

    @property
    def scan_pid(self):
        scan_pid = int(os.environ.get(self.scan_process_var_name, 0))
        if scan_pid == 0:
            scan_pid = os.getpid()
            # if scan PID is not set, set it to the current PID
            os.environ[self.scan_process_var_name] = str(scan_pid)
        return scan_pid

    def update_scan_pid(self):
        os.environ[self.scan_process_var_name] = str(os.getpid())

    def cleanup(self):
        with suppress(Exception):
            if self.is_main_process:
                with suppress(KeyError):
                    del os.environ[self.main_process_var_name]
                with suppress(KeyError):
                    del os.environ[self.scan_process_var_name]


SHARED_INTERPRETER_STATE = SharedInterpreterState()



from bbot.modules.templates.subdomain_enum import subdomain_enum


class subdomaincenter(subdomain_enum):
    flags = ["subdomain-enum", "passive", "safe"]
    watched_events = ["DNS_NAME"]
    produced_events = ["DNS_NAME"]
    meta = {
        "description": "Query subdomain.center's API for subdomains",
        "created_date": "2023-07-26",
        "author": "@TheTechromancer",
    }

    base_url = "https://api.subdomain.center"

    async def request_url(self, query):
        url = f"{self.base_url}/?domain={self.helpers.quote(query)}"
        response = await self.api_request(url)
        return response

    async def parse_results(self, r, query):
        results = set()
        json = r.json()
        if json and isinstance(json, list):
            results = set(json)
        return results



from .base import ModuleTestBase


class TestDNSDumpster(ModuleTestBase):
    async def setup_after_prep(self, module_test):
        module_test.httpx_mock.add_response(
            url="https://dnsdumpster.com",
            content=b"""<form data-form-id="mainform" class="mb-6" hx-post="https://api.dnsdumpster.com/htmld/" hx-target="#results" hx-headers='{"Authorization": "eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJzdWIiOjAsImlhdCI6MTc1OTAxODczOCwiZXhwIjoxNzU5MDE5NjM4LCJkYXRhIjoiZmMxMDcwOTVjYmRjN2Y5YjU1ZWJiM2ZlZGViNWQ5Y2M5MWU1NmEzNGEwYzliNzM5ZjRlYzg2Mjk4MmM0ZDI5YSIsIm1lbWJlcl9zdGF0dXMiOiJmcmVlIn0.7NWBC6TFSaDZH-_VKqDoXqv3nH4a1k30NUxrijg1KqI"}'><div class="form-group">""",
        )
        module_test.httpx_mock.add_response(
            url="https://api.dnsdumpster.com/htmld/",
            content=b"asdf.blacklanternsecurity.com",
        )

    def check(self, module_test, events):
        assert any(e.data == "asdf.blacklanternsecurity.com" for e in events), "Failed to detect subdomain"



from .base import ModuleTestBase


class TestFingerprintx(ModuleTestBase):
    targets = ["127.0.0.1:8888"]

    def check(self, module_test, events):
        assert any(
            event.type == "PROTOCOL"
            and event.host == module_test.scan.helpers.make_ip_type("127.0.0.1")
            and event.port == 8888
            and event.data["protocol"] == "HTTP"
            for event in events
        ), "HTTP protocol not detected"



from .base import ModuleTestBase


class TestCRT(ModuleTestBase):
    async def setup_after_prep(self, module_test):
        module_test.module.abort_if = lambda e: False
        for t in self.targets:
            module_test.httpx_mock.add_response(
                url="https://crt.sh?q=%25.blacklanternsecurity.com&output=json",
                json=[{"id": 1, "name_value": "asdf.blacklanternsecurity.com\nzzzz.blacklanternsecurity.com"}],
            )

    def check(self, module_test, events):
        assert any(e.data == "asdf.blacklanternsecurity.com" for e in events), "Failed to detect subdomain"
        assert any(e.data == "zzzz.blacklanternsecurity.com" for e in events), "Failed to detect subdomain"



import re
import ast
import sys
import atexit
import pickle
import logging
import importlib
import omegaconf
import traceback
from copy import copy
from pathlib import Path
from omegaconf import OmegaConf
from contextlib import suppress

from bbot.core import CORE
from bbot.errors import BBOTError
from bbot.logger import log_to_stderr

from .flags import flag_descriptions
from .shared_deps import SHARED_DEPS
from .helpers.misc import (
    list_files,
    sha1,
    search_dict_by_key,
    search_format_dict,
    make_table,
    os_platform,
    mkdir,
)


log = logging.getLogger("bbot.module_loader")

bbot_code_dir = Path(__file__).parent.parent


class ModuleLoader:
    """
    Main class responsible for preloading BBOT modules.

    This class is in charge of preloading modules to determine their dependencies.
    Once dependencies are identified, they are installed before the actual module is imported.
    This ensures that all requisite libraries and components are available for the module to function correctly.
    """

    default_module_dir = bbot_code_dir / "modules"

    module_dir_regex = re.compile(r"^[a-z][a-z0-9_]*$")

    # if a module consumes these event types, automatically assume these dependencies
    default_module_deps = {"HTTP_RESPONSE": "httpx", "URL": "httpx", "SOCIAL": "social"}

    def __init__(self):
        self.core = CORE

        self._shared_deps = dict(SHARED_DEPS)

        self.__preloaded = {}
        self._configs = {}
        self.flag_choices = set()
        self.all_module_choices = set()
        self.scan_module_choices = set()
        self.output_module_choices = set()
        self.internal_module_choices = set()

        self._preload_cache = None

        self._module_dirs = set()
        self._module_dirs_preloaded = set()
        self.add_module_dir(self.default_module_dir)

        # save preload cache before exiting
        atexit.register(self.save_preload_cache)

    def copy(self):
        module_loader_copy = copy(self)
        module_loader_copy.__preloaded = dict(self.__preloaded)
        return module_loader_copy

    @property
    def preload_cache_file(self):
        return self.core.cache_dir / "module_preload_cache"

    @property
    def module_dirs(self):
        return self._module_dirs

    def add_module_dir(self, module_dir):
        module_dir = Path(module_dir).resolve()
        if module_dir in self._module_dirs:
            log.debug(f'Already added custom module dir "{module_dir}"')
            return
        if not module_dir.is_dir():
            log.warning(f'Failed to add custom module dir "{module_dir}", please make sure it exists')
            return
        new_module_dirs = set()
        for _module_dir in self.get_recursive_dirs(module_dir):
            _module_dir = Path(_module_dir).resolve()
            if _module_dir not in self._module_dirs:
                self._module_dirs.add(_module_dir)
                new_module_dirs.add(_module_dir)
        self.preload(module_dirs=new_module_dirs)

    def file_filter(self, file):
        file = file.resolve()
        for part in file.parts:
            if part.endswith("_submodules") or part == "templates":
                return False
        return file.suffix.lower() == ".py" and file.stem not in ["base", "__init__"]

    def preload(self, module_dirs=None):
        """Preloads all BBOT modules.

        This function recursively iterates through each file in the module directories
        and preloads each BBOT module to gather its meta-information and dependencies.

        Args:
            module_dir (str or Path): Directory containing BBOT modules to be preloaded.

        Returns:
            dict: A dictionary where keys are the names of the preloaded modules and
            values are their respective preloaded data.

        Examples:
            >>> preload("/path/to/bbot_modules/")
            {
                "module1": {...},
                "module2": {...},
                ...
            }
        """
        new_modules = False
        if module_dirs is None:
            module_dirs = self.module_dirs

        for module_dir in module_dirs:
            if module_dir in self._module_dirs_preloaded:
                log.debug(f"Already preloaded modules from {module_dir}")
                continue

            log.debug(f"Preloading modules from {module_dir}")
            new_modules = True
            for module_file in list_files(module_dir, filter=self.file_filter):
                module_name = module_file.stem
                module_file = module_file.resolve()

                # try to load from cache
                module_cache_key = (str(module_file), tuple(module_file.stat()))
                preloaded = self.preload_cache.get(module_name, {})
                cache_key = preloaded.get("cache_key", ())
                if preloaded and module_cache_key == cache_key:
                    log.debug(f"Preloading {module_name} from cache")
                else:
                    log.debug(f"Preloading {module_name} from disk")
                    if module_dir.name == "modules":
                        namespace = "bbot.modules"
                    else:
                        namespace = f"bbot.modules.{module_dir.name}"
                    try:
                        preloaded = self.preload_module(module_file)
                        if preloaded is None:
                            continue
                        module_type = "scan"
                        if module_dir.name in ("output", "internal"):
                            module_type = str(module_dir.name)

                        # derive module dependencies from watched event types (only for scan modules)
                        if module_type == "scan":
                            for event_type in preloaded["watched_events"]:
                                if event_type in self.default_module_deps:
                                    deps_modules = set(preloaded.get("deps", {}).get("modules", []))
                                    deps_modules.add(self.default_module_deps[event_type])
                                    preloaded["deps"]["modules"] = sorted(deps_modules)

                        preloaded["type"] = module_type
                        preloaded["namespace"] = namespace
                        preloaded["cache_key"] = module_cache_key

                    except Exception:
                        log_to_stderr(f"Error preloading {module_file}\n\n{traceback.format_exc()}", level="CRITICAL")
                        log_to_stderr(f"Error in {module_file.name}", level="CRITICAL")
                        sys.exit(1)

                self.all_module_choices.add(module_name)
                module_type = preloaded.get("type", "scan")
                if module_type == "scan":
                    self.scan_module_choices.add(module_name)
                elif module_type == "output":
                    self.output_module_choices.add(module_name)
                elif module_type == "internal":
                    self.internal_module_choices.add(module_name)

                flags = preloaded.get("flags", [])
                self.flag_choices.update(set(flags))

                self.__preloaded[module_name] = preloaded
                config = OmegaConf.create(preloaded.get("config", {}))
                self._configs[module_name] = config

            self._module_dirs_preloaded.add(module_dir)

        # update default config with module defaults
        module_config = omegaconf.OmegaConf.create(
            {
                "modules": self.configs(),
            }
        )
        self.core.merge_default(module_config)

        return new_modules

    @property
    def preload_cache(self):
        if self._preload_cache is None:
            self._preload_cache = {}
            if self.preload_cache_file.is_file():
                with suppress(Exception):
                    with open(self.preload_cache_file, "rb") as f:
                        self._preload_cache = pickle.load(f)
        return self._preload_cache

    @preload_cache.setter
    def preload_cache(self, value):
        self._preload_cache = value
        mkdir(self.preload_cache_file.parent)
        with open(self.preload_cache_file, "wb") as f:
            pickle.dump(self._preload_cache, f)

    def save_preload_cache(self):
        self.preload_cache = self.__preloaded

    @property
    def _preloaded(self):
        return self.__preloaded

    def get_recursive_dirs(self, *dirs):
        dirs = {Path(d).resolve() for d in dirs}
        for d in list(dirs):
            if not d.is_dir():
                continue
            for p in d.iterdir():
                if p.is_dir() and self.module_dir_regex.match(p.name):
                    dirs.update(self.get_recursive_dirs(p))
        return dirs

    def preloaded(self, type=None):
        preloaded = {}
        if type is not None:
            preloaded = {k: v for k, v in self._preloaded.items() if self.check_type(k, type)}
        else:
            preloaded = dict(self._preloaded)
        return preloaded

    def configs(self, type=None):
        configs = {}
        if type is not None:
            configs = {k: v for k, v in self._configs.items() if self.check_type(k, type)}
        else:
            configs = dict(self._configs)
        return OmegaConf.create(configs)

    def find_and_replace(self, **kwargs):
        self.__preloaded = search_format_dict(self.__preloaded, **kwargs)
        self._shared_deps = search_format_dict(self._shared_deps, **kwargs)

    def check_type(self, module, type):
        return self._preloaded[module]["type"] == type

    def preload_module(self, module_file):
        """
        Preloads a BBOT module to gather its meta-information and dependencies.

        This function reads a BBOT module file, extracts its attributes such as
        events watched and produced, flags, meta-information, and dependencies.

        Args:
            module_file (str): Path to the BBOT module file.

        Returns:
            dict: A dictionary containing meta-information and dependencies for the module.

        Examples:
            >>> preload_module("bbot/modules/wappalyzer.py")
            {
                "watched_events": [
                    "HTTP_RESPONSE"
                ],
                "produced_events": [
                    "TECHNOLOGY"
                ],
                "flags": [
                    "active",
                    "safe",
                    "web-basic",
                    "web-thorough"
                ],
                "meta": {
                    "description": "Extract technologies from web responses"
                },
                "config": {},
                "options_desc": {},
                "hash": "d5a88dd3866c876b81939c920bf4959716e2a374",
                "deps": {
                    "modules": [
                        "httpx"
                    ]
                    "pip": [
                        "python-Wappalyzer~=0.3.1"
                    ],
                    "pip_constraints": [],
                    "shell": [],
                    "apt": [],
                    "ansible": []
                },
                "sudo": false
            }
        """
        watched_events = set()
        produced_events = set()
        flags = set()
        meta = {}
        deps_modules = set()
        deps_pip = []
        deps_pip_constraints = []
        deps_shell = []
        deps_apt = []
        deps_common = []
        ansible_tasks = []
        config = {}
        options_desc = {}
        python_code = open(module_file).read()
        # take a hash of the code so we can keep track of when it changes
        module_hash = sha1(python_code).hexdigest()
        parsed_code = ast.parse(python_code)

        # discard if the module isn't a valid BBOT module
        is_bbot_module = False
        for root_element in parsed_code.body:
            if type(root_element) == ast.ClassDef:
                for class_attr in root_element.body:
                    if type(class_attr) == ast.Assign and any(
                        target.id in ("watched_events", "produced_events") for target in class_attr.targets
                    ):
                        is_bbot_module = True
                        break

        if not is_bbot_module:
            log.debug(f"Skipping {module_file} as it is not a valid BBOT module")
            return

        for root_element in parsed_code.body:
            # look for classes
            if type(root_element) == ast.ClassDef:
                for class_attr in root_element.body:
                    # class attributes that are dictionaries
                    if type(class_attr) == ast.Assign and type(class_attr.value) == ast.Dict:
                        # module options
                        if any(target.id == "options" for target in class_attr.targets):
                            config.update(ast.literal_eval(class_attr.value))
                        # module options
                        elif any(target.id == "options_desc" for target in class_attr.targets):
                            options_desc.update(ast.literal_eval(class_attr.value))
                        # module metadata
                        elif any(target.id == "meta" for target in class_attr.targets):
                            meta = ast.literal_eval(class_attr.value)

                    # class attributes that are lists
                    if type(class_attr) == ast.Assign and type(class_attr.value) == ast.List:
                        # flags
                        if any(target.id == "flags" for target in class_attr.targets):
                            for flag in class_attr.value.elts:
                                if type(flag.value) == str:
                                    flags.add(flag.value)
                        # watched events
                        elif any(target.id == "watched_events" for target in class_attr.targets):
                            for event_type in class_attr.value.elts:
                                if type(event_type.value) == str:
                                    watched_events.add(event_type.value)
                        # produced events
                        elif any(target.id == "produced_events" for target in class_attr.targets):
                            for event_type in class_attr.value.elts:
                                if type(event_type.value) == str:
                                    produced_events.add(event_type.value)

                        # bbot module dependencies
                        elif any(target.id == "deps_modules" for target in class_attr.targets):
                            for dep_module in class_attr.value.elts:
                                if type(dep_module.value) == str:
                                    deps_modules.add(dep_module.value)
                        # python dependencies
                        elif any(target.id == "deps_pip" for target in class_attr.targets):
                            for dep_pip in class_attr.value.elts:
                                if type(dep_pip.value) == str:
                                    deps_pip.append(dep_pip.value)
                        elif any(target.id == "deps_pip_constraints" for target in class_attr.targets):
                            for dep_pip in class_attr.value.elts:
                                if type(dep_pip.value) == str:
                                    deps_pip_constraints.append(dep_pip.value)
                        # apt dependencies
                        elif any(target.id == "deps_apt" for target in class_attr.targets):
                            for dep_apt in class_attr.value.elts:
                                if type(dep_apt.value) == str:
                                    deps_apt.append(dep_apt.value)
                        # bash dependencies
                        elif any(target.id == "deps_shell" for target in class_attr.targets):
                            for dep_shell in class_attr.value.elts:
                                deps_shell.append(ast.literal_eval(dep_shell))
                        # ansible playbook
                        elif any(target.id == "deps_ansible" for target in class_attr.targets):
                            ansible_tasks = ast.literal_eval(class_attr.value)
                        # shared/common module dependencies
                        elif any(target.id == "deps_common" for target in class_attr.targets):
                            for dep_common in class_attr.value.elts:
                                if type(dep_common.value) == str:
                                    deps_common.append(dep_common.value)

        for task in ansible_tasks:
            if "become" not in task:
                task["become"] = False
            # don't sudo brew
            elif os_platform() == "darwin" and ("package" in task and task.get("become", False) is True):
                task["become"] = False

        preloaded_data = {
            "path": str(module_file.resolve()),
            "watched_events": sorted(watched_events),
            "produced_events": sorted(produced_events),
            "flags": sorted(flags),
            "meta": meta,
            "config": config,
            "options_desc": options_desc,
            "hash": module_hash,
            "deps": {
                "modules": sorted(deps_modules),
                "pip": deps_pip,
                "pip_constraints": deps_pip_constraints,
                "shell": deps_shell,
                "apt": deps_apt,
                "ansible": ansible_tasks,
                "common": deps_common,
            },
            "sudo": len(deps_apt) > 0,
        }
        ansible_task_list = list(ansible_tasks)
        for dep_common in deps_common:
            try:
                ansible_task_list.extend(self._shared_deps[dep_common])
            except KeyError:
                common_choices = ",".join(self._shared_deps)
                raise BBOTError(
                    f'Error while preloading module "{module_file}": No shared dependency named "{dep_common}" (choices: {common_choices})'
                )
        for ansible_task in ansible_task_list:
            if any(x is True for x in search_dict_by_key("become", ansible_task)) or any(
                x is True for x in search_dict_by_key("ansible_become", ansible_tasks)
            ):
                preloaded_data["sudo"] = True
        return preloaded_data

    def load_modules(self, module_names):
        modules = {}
        for module_name in module_names:
            try:
                module = self.load_module(module_name)
            except ModuleNotFoundError as e:
                raise BBOTError(
                    f"Error loading module {module_name}: {e}. You may have leftover artifacts from an older version of BBOT. Try deleting/renaming your '~/.bbot' directory."
                ) from e
            modules[module_name] = module
        return modules

    def load_module(self, module_name):
        """Loads a BBOT module by its name.

        Imports the module from its namespace, locates its class, and returns it.
        Identifies modules based on the presence of `watched_events` and `produced_events` attributes.

        Args:
            module_name (str): The name of the module to load.

        Returns:
            object: The loaded module class object.

        Examples:
            >>> module = load_module("example_module")
            >>> isinstance(module, object)
            True
        """
        preloaded = self._preloaded[module_name]
        namespace = preloaded["namespace"]
        try:
            module_path = preloaded["path"]
        except KeyError:
            module_path = preloaded["cache_key"][0]
        full_namespace = f"{namespace}.{module_name}"

        spec = importlib.util.spec_from_file_location(full_namespace, module_path)
        module = importlib.util.module_from_spec(spec)
        sys.modules[full_namespace] = module
        spec.loader.exec_module(module)

        # for every top-level variable in the .py file
        for variable in module.__dict__.keys():
            # get its value
            value = getattr(module, variable)
            with suppress(AttributeError):
                # if it has watched_events and produced_events
                if all(
                    type(a) == list
                    for a in (getattr(value, "watched_events", None), getattr(value, "produced_events", None))
                ):
                    # and if its variable name matches its filename
                    if value.__name__.lower() == module_name.lower():
                        value._name = module_name
                        # then we have a module
                        return value

    def check_dependency(self, event_type, modname, produced):
        if event_type not in produced:
            return False
        if produced[event_type] == {modname}:
            return False
        return True

    @staticmethod
    def add_or_create(d, k, *items):
        try:
            d[k].update(set(items))
        except KeyError:
            d[k] = set(items)

    def modules_table(self, modules=None, mod_type=None, include_author=False, include_created_date=False):
        """Generates a table of module information.

        Constructs a table to display information such as module name, type, and event details.

        Args:
            modules (list, optional): List of module names to include in the table.
            mod_type (str, optional): Type of modules to include ('scan', 'output', 'internal').

        Returns:
            str: A formatted table string.

        Examples:
            >>> print(modules_table(["portscan"]))
            +----------+--------+-----------------+------------------------------+-------------------------------+----------------------+-------------------+
            | Module   | Type   | Needs API Key   | Description                  | Flags                         | Consumed Events      | Produced Events   |
            +==========+========+=================+==============================+===============================+======================+===================+
            | portscan | scan   | No              | Execute port scans           | active, aggressive, portscan, | DNS_NAME, IP_ADDRESS | OPEN_TCP_PORT     |
            |          |        |                 |                              | web-thorough                  |                      |                   |
            +----------+--------+-----------------+------------------------------+-------------------------------+----------------------+-------------------+
        """

        table = []
        header = ["Module", "Type", "Needs API Key", "Description", "Flags", "Consumed Events", "Produced Events"]
        if include_author:
            header.append("Author")
        if include_created_date:
            header.append("Created Date")
        maxcolwidths = [20, 10, 5, 30, 30, 20, 20]
        for module_name, preloaded in self.filter_modules(modules, mod_type):
            module_type = preloaded["type"]
            consumed_events = sorted(preloaded.get("watched_events", []))
            produced_events = sorted(preloaded.get("produced_events", []))
            flags = sorted(preloaded.get("flags", []))
            api_key_required = ""
            meta = preloaded.get("meta", {})
            api_key_required = "Yes" if meta.get("auth_required", False) else "No"
            description = meta.get("description", "")
            row = [
                module_name,
                module_type,
                api_key_required,
                description,
                ", ".join(flags),
                ", ".join(consumed_events),
                ", ".join(produced_events),
            ]
            if include_author:
                author = meta.get("author", "")
                row.append(author)
            if include_created_date:
                created_date = meta.get("created_date", "")
                row.append(created_date)
            table.append(row)
        return make_table(table, header, maxcolwidths=maxcolwidths)

    def modules_options(self, modules=None, mod_type=None):
        """
        Return a list of module options
        """
        modules_options = {}
        for module_name, preloaded in self.filter_modules(modules, mod_type):
            modules_options[module_name] = []
            module_options = preloaded["config"]
            module_options_desc = preloaded["options_desc"]
            for k, v in sorted(module_options.items(), key=lambda x: x[0]):
                option_name = f"modules.{module_name}.{k}"
                option_type = type(v).__name__
                option_description = module_options_desc[k]
                modules_options[module_name].append((option_name, option_type, option_description, str(v)))
        return modules_options

    def modules_options_table(self, modules=None, mod_type=None):
        table = []
        header = ["Config Option", "Type", "Description", "Default"]
        for module_options in self.modules_options(modules, mod_type).values():
            table += module_options
        return make_table(table, header)

    def flags(self, flags=None):
        _flags = {}
        for module_name, preloaded in self.preloaded().items():
            for flag in preloaded.get("flags", []):
                if not flags or flag in flags:
                    try:
                        _flags[flag].add(module_name)
                    except KeyError:
                        _flags[flag] = {module_name}

        _flags = sorted(_flags.items(), key=lambda x: x[0])
        _flags = sorted(_flags, key=lambda x: len(x[-1]), reverse=True)
        return _flags

    def flags_table(self, flags=None):
        table = []
        header = ["Flag", "# Modules", "Description", "Modules"]
        maxcolwidths = [20, 5, 40, 80]
        _flags = self.flags(flags=flags)
        for flag, modules in _flags:
            description = flag_descriptions.get(flag, "")
            table.append([flag, f"{len(modules)}", description, ", ".join(sorted(modules))])
        return make_table(table, header, maxcolwidths=maxcolwidths)

    def events(self):
        consuming_events = {}
        producing_events = {}
        for module_name, preloaded in self.preloaded().items():
            consumed = preloaded.get("watched_events", [])
            produced = preloaded.get("produced_events", [])
            for c in consumed:
                try:
                    consuming_events[c].add(module_name)
                except KeyError:
                    consuming_events[c] = {module_name}
            for c in produced:
                try:
                    producing_events[c].add(module_name)
                except KeyError:
                    producing_events[c] = {module_name}
        return consuming_events, producing_events

    def events_table(self):
        table = []
        header = ["Event Type", "# Consuming Modules", "# Producing Modules", "Consuming Modules", "Producing Modules"]
        consuming_events, producing_events = self.events()
        all_event_types = sorted(set(consuming_events).union(set(producing_events)))
        for e in all_event_types:
            consuming = sorted(consuming_events.get(e, []))
            producing = sorted(producing_events.get(e, []))
            table.append([e, len(consuming), len(producing), ", ".join(consuming), ", ".join(producing)])
        return make_table(table, header)

    def filter_modules(self, modules=None, mod_type=None):
        if modules is None:
            module_list = list(self.preloaded(type=mod_type).items())
        else:
            module_list = [(m, self._preloaded[m]) for m in modules]
        module_list.sort(key=lambda x: x[0])
        module_list.sort(key=lambda x: "passive" in x[-1]["flags"])
        module_list.sort(key=lambda x: x[-1]["type"], reverse=True)
        return module_list

    def ensure_config_files(self):
        files = self.core.files_config
        mkdir(files.config_dir)

        comment_notice = (
            "# NOTICE: THESE ENTRIES ARE COMMENTED BY DEFAULT\n"
            + "# Please be sure to uncomment when inserting API keys, etc.\n"
        )

        config_obj = OmegaConf.to_object(self.core.default_config)

        # ensure bbot.yml
        if not files.config_filename.exists():
            log_to_stderr(f"Creating BBOT config at {files.config_filename}")
            no_secrets_config = self.core.no_secrets_config(config_obj)
            yaml = OmegaConf.to_yaml(no_secrets_config)
            yaml = comment_notice + "\n".join(f"# {line}" for line in yaml.splitlines())
            with open(str(files.config_filename), "w") as f:
                f.write(yaml)

        # ensure secrets.yml
        if not files.secrets_filename.exists():
            log_to_stderr(f"Creating BBOT secrets at {files.secrets_filename}")
            secrets_only_config = self.core.secrets_only_config(config_obj)
            yaml = OmegaConf.to_yaml(secrets_only_config)
            yaml = comment_notice + "\n".join(f"# {line}" for line in yaml.splitlines())
            with open(str(files.secrets_filename), "w") as f:
                f.write(yaml)
            files.secrets_filename.chmod(0o600)


MODULE_LOADER = ModuleLoader()



from pathlib import Path

from bbot.modules.templates.sql import SQLTemplate


class SQLite(SQLTemplate):
    watched_events = ["*"]
    meta = {
        "description": "Output scan data to a SQLite database",
        "created_date": "2024-11-07",
        "author": "@TheTechromancer",
    }
    options = {
        "database": "",
    }
    options_desc = {
        "database": "The path to the sqlite database file",
    }
    deps_pip = ["sqlmodel", "aiosqlite"]

    async def setup(self):
        db_file = self.config.get("database", "")
        if not db_file:
            db_file = self.scan.home / "output.sqlite"
        db_file = Path(db_file)
        if not db_file.is_absolute():
            db_file = self.scan.home / db_file
        self.db_file = db_file
        self.db_file.parent.mkdir(parents=True, exist_ok=True)
        return await super().setup()

    def connection_string(self, mask_password=False):
        return f"sqlite+aiosqlite:///{self.db_file}"



from bbot.modules.templates.gitlab import GitLabBaseModule


class gitlab_onprem(GitLabBaseModule):
    watched_events = ["HTTP_RESPONSE", "TECHNOLOGY", "SOCIAL"]
    produced_events = [
        "TECHNOLOGY",
        "SOCIAL",
        "CODE_REPOSITORY",
        "FINDING",
    ]
    flags = ["active", "safe", "code-enum"]
    meta = {
        "description": "Detect self-hosted GitLab instances and query them for repositories",
        "created_date": "2024-03-11",
        "author": "@TheTechromancer",
    }

    # Optional GitLab access token (only required for gitlab.com, but still
    # supported for on-prem installations that expose private projects).
    options = {"api_key": ""}
    options_desc = {"api_key": "GitLab access token (for self-hosted instances only)"}

    # Allow accepting events slightly beyond configured max distance so we can
    # discover repos on neighbouring infrastructure.
    scope_distance_modifier = 2

    async def handle_event(self, event):
        if event.type == "HTTP_RESPONSE":
            await self.handle_http_response(event)
        elif event.type == "TECHNOLOGY":
            await self.handle_technology(event)
        elif event.type == "SOCIAL":
            await self.handle_social(event)

    async def filter_event(self, event):
        # only accept out-of-scope SOCIAL events
        if event.type == "HTTP_RESPONSE":
            if event.scope_distance > self.scan.scope_search_distance:
                return False, "event is out of scope distance"
        elif event.type == "TECHNOLOGY":
            if not event.data["technology"].lower().startswith("gitlab"):
                return False, "technology is not gitlab"
            if not self.helpers.is_ip(event.host) and self.helpers.tldextract(event.host).domain == "gitlab":
                return False, "gitlab instance is not self-hosted"
        elif event.type == "SOCIAL":
            if event.data["platform"] != "gitlab":
                return False, "platform is not gitlab"
            _, domain = self.helpers.split_domain(event.host)
            if domain in self.saas_domains:
                return False, "gitlab instance is not self-hosted"
        return True

    async def handle_http_response(self, event):
        """Identify GitLab servers from HTTP responses."""
        headers = event.data.get("header", {})
        if "x_gitlab_meta" in headers:
            url = event.parsed_url._replace(path="/").geturl()
            await self.emit_event(
                {"host": str(event.host), "technology": "GitLab", "url": url},
                "TECHNOLOGY",
                parent=event,
                context=f"{{module}} detected {{event.type}}: GitLab at {url}",
            )
            description = f"GitLab server at {event.host}"
            await self.emit_event(
                {"host": str(event.host), "description": description},
                "FINDING",
                parent=event,
                context=f"{{module}} detected {{event.type}}: {description}",
            )

    async def handle_technology(self, event):
        """Enumerate projects & groups once we know a host is GitLab."""
        base_url = self.get_base_url(event)

        # Projects owned by the authenticated user (or public projects if no
        # authentication).
        projects_url = self.helpers.urljoin(base_url, "api/v4/projects?simple=true")
        await self.handle_projects_url(projects_url, event)

        # Group enumeration.
        groups_url = self.helpers.urljoin(base_url, "api/v4/groups?simple=true")
        await self.handle_groups_url(groups_url, event)



from bbot.modules.base import BaseModule


class aspnet_bin_exposure(BaseModule):
    watched_events = ["URL"]
    produced_events = ["VULNERABILITY"]
    flags = ["active", "safe", "web-thorough"]
    meta = {
        "description": "Check for ASP.NET Security Feature Bypasses (CVE-2023-36899 and CVE-2023-36560)",
        "created_date": "2025-01-28",
        "author": "@liquidsec",
    }

    in_scope_only = True
    test_dlls = [
        "Telerik.Web.UI.dll",
        "Newtonsoft.Json.dll",
        "System.Net.Http.dll",
        "EntityFramework.dll",
        "AjaxControlToolkit.dll",
    ]

    @staticmethod
    def normalize_url(url):
        return str(url.rstrip("/") + "/").lower()

    def _incoming_dedup_hash(self, event):
        return hash(self.normalize_url(event.data))

    async def handle_event(self, event):
        normalized_url = self.normalize_url(event.data)
        for test_dll in self.test_dlls:
            for technique in ["b/(S(X))in/###DLL_PLACEHOLDER###/(S(X))/", "(S(X))/b/(S(X))in/###DLL_PLACEHOLDER###"]:
                test_url = f"{normalized_url}{technique.replace('###DLL_PLACEHOLDER###', test_dll)}"
                self.debug(f"Sending test URL: [{test_url}]")
                kwargs = {"method": "GET", "allow_redirects": False, "timeout": 10}
                test_result = await self.helpers.request(test_url, **kwargs)
                if test_result:
                    if test_result.status_code == 200 and (
                        "content-type" in test_result.headers
                        and "application/x-msdownload" in test_result.headers["content-type"]
                    ):
                        self.debug(
                            f"Got positive result for probe with test url: [{test_url}]. Status Code: [{test_result.status_code}] Content Length: [{len(test_result.content)}]"
                        )

                        if test_result.status_code == 200 and (
                            "content-type" in test_result.headers
                            and "application/x-msdownload" in test_result.headers["content-type"]
                        ):
                            confirm_url = (
                                f"{normalized_url}{technique.replace('###DLL_PLACEHOLDER###', 'oopsnotarealdll.dll')}"
                            )
                            confirm_result = await self.helpers.request(confirm_url, **kwargs)

                            if confirm_result and (
                                confirm_result.status_code != 200
                                or not (
                                    "content-type" in confirm_result.headers
                                    and "application/x-msdownload" in confirm_result.headers["content-type"]
                                )
                            ):
                                description = f"IIS Bin Directory DLL Exposure. Detection Url: [{test_url}]"
                                await self.emit_event(
                                    {
                                        "severity": "HIGH",
                                        "host": str(event.host),
                                        "url": normalized_url,
                                        "description": description,
                                    },
                                    "VULNERABILITY",
                                    event,
                                    context="{module} detected IIS Bin Directory DLL Exposure vulnerability",
                                )
                                return True

    async def filter_event(self, event):
        if "dir" in event.tags:
            return True
        return False



import json
from functools import partial
from bbot.modules.base import BaseModule


class trufflehog(BaseModule):
    watched_events = ["CODE_REPOSITORY", "FILESYSTEM", "HTTP_RESPONSE", "RAW_TEXT"]
    produced_events = ["FINDING", "VULNERABILITY"]
    flags = ["passive", "safe", "code-enum"]
    meta = {
        "description": "TruffleHog is a tool for finding credentials",
        "created_date": "2024-03-12",
        "author": "@domwhewell-sage",
    }

    options = {
        "version": "3.90.8",
        "config": "",
        "only_verified": True,
        "concurrency": 8,
        "deleted_forks": False,
    }
    options_desc = {
        "version": "trufflehog version",
        "config": "File path or URL to YAML trufflehog config",
        "only_verified": "Only report credentials that have been verified",
        "concurrency": "Number of concurrent workers",
        "deleted_forks": "Scan for deleted github forks. WARNING: This is SLOW. For a smaller repository, this process can take 20 minutes. For a larger repository, it could take hours.",
    }
    deps_ansible = [
        {
            "name": "Download trufflehog",
            "unarchive": {
                "src": "https://github.com/trufflesecurity/trufflehog/releases/download/v#{BBOT_MODULES_TRUFFLEHOG_VERSION}/trufflehog_#{BBOT_MODULES_TRUFFLEHOG_VERSION}_#{BBOT_OS_PLATFORM}_#{BBOT_CPU_ARCH}.tar.gz",
                "include": "trufflehog",
                "dest": "#{BBOT_TOOLS}",
                "remote_src": True,
            },
        }
    ]

    scope_distance_modifier = 2

    async def setup_deps(self):
        self.config_file = self.config.get("config", "")
        if self.config_file:
            self.config_file = await self.helpers.wordlist(self.config_file)
        return True

    async def setup(self):
        self.verified = self.config.get("only_verified", True)
        self.concurrency = int(self.config.get("concurrency", 8))

        self.deleted_forks = self.config.get("deleted_forks", False)
        self.github_token = ""
        if self.deleted_forks:
            self.warning(
                "Deleted forks is enabled. Scanning for deleted forks is slooooooowwwww. For a smaller repository, this process can take 20 minutes. For a larger repository, it could take hours."
            )
            for module_name in ("github", "github_codesearch", "github_org", "git_clone"):
                module_config = self.scan.config.get("modules", {}).get(module_name, {})
                api_key = module_config.get("api_key", "")
                if api_key:
                    self.github_token = api_key
                    break

            # soft-fail if we don't have a github token as well
            if not self.github_token:
                self.deleted_forks = False
                return None, "A github api_key must be provided to the github modules for deleted forks to be scanned"
        return True

    async def filter_event(self, event):
        if event.type == "CODE_REPOSITORY":
            if self.deleted_forks:
                if "git" not in event.tags:
                    return False, "Module only accepts git CODE_REPOSITORY events"
                if "github" not in event.data["url"]:
                    return False, "Module only accepts github CODE_REPOSITORY events"
            else:
                return False, "Deleted forks is not enabled"
        else:
            if "unarchived-folder" in event.tags:
                return False, "Not accepting unarchived-folder events"
        return True

    async def handle_event(self, event):
        description = ""
        if isinstance(event.data, dict):
            description = event.data.get("description", "")

        if event.type == "CODE_REPOSITORY":
            path = event.data["url"]
            module = "github-experimental"
        elif event.type == "FILESYSTEM":
            path = event.data["path"]
            if "git" in event.tags:
                module = "git"
            elif "docker" in event.tags:
                module = "docker"
            elif "postman" in event.tags:
                module = "postman"
            else:
                module = "filesystem"
        elif event.type in ("HTTP_RESPONSE", "RAW_TEXT"):
            module = "filesystem"
            file_data = event.raw_response if event.type == "HTTP_RESPONSE" else event.data
            # write the response to a tempfile
            # this is necessary because trufflehog doesn't yet support reading from stdin
            # https://github.com/trufflesecurity/trufflehog/issues/162
            path = self.helpers.tempfile(file_data, pipe=False)

        if event.type == "CODE_REPOSITORY":
            host = event.host
        else:
            host = str(event.parent.host)
        async for (
            decoder_name,
            detector_name,
            raw_result,
            rawv2_result,
            verified,
            source_metadata,
        ) in self.execute_trufflehog(module, path):
            verified_str = "Verified" if verified else "Possible"
            finding_type = "VULNERABILITY" if verified else "FINDING"
            data = {
                "description": f"{verified_str} Secret Found. Detector Type: [{detector_name}] Decoder Type: [{decoder_name}] Details: [{source_metadata}]",
            }
            if host:
                data["host"] = host
            if finding_type == "VULNERABILITY":
                data["severity"] = "High"
            if description:
                data["description"] += f" Description: [{description}]"
            data["description"] += f" Raw result: [{raw_result}]"
            if rawv2_result:
                data["description"] += f" RawV2 result: [{rawv2_result}]"
            await self.emit_event(
                data,
                finding_type,
                event,
                context=f'{{module}} searched {event.type} using "{module}" method and found {verified_str.lower()} secret ({{event.type}}): {raw_result}',
            )

        # clean up the tempfile when we're done with it
        if event.type in ("HTTP_RESPONSE", "RAW_TEXT"):
            path.unlink(missing_ok=True)

    async def execute_trufflehog(self, module, path=None, string=None):
        command = [
            "trufflehog",
            "--json",
            "--no-update",
        ]
        if self.verified:
            command.append("--only-verified")
        if self.config_file:
            command.append("--config=" + str(self.config_file))
        command.append("--concurrency=" + str(self.concurrency))
        if module == "git":
            command.append("git")
            command.append("file://" + path)
        elif module == "docker":
            command.append("docker")
            command.append("--image=file://" + path)
        elif module == "postman":
            command.append("postman")
            command.append("--workspace-paths=" + path)
        elif module == "filesystem":
            command.append("filesystem")
            command.append(path)
        elif module == "github-experimental":
            command.append("github-experimental")
            command.append("--repo=" + path)
            command.append("--object-discovery")
            command.append("--delete-cached-data")
            command.append("--token=" + self.github_token)

        stats_file = self.helpers.tempfile_tail(callback=partial(self.log_trufflehog_status, path))
        try:
            with open(stats_file, "w") as stats_fh:
                async for line in self.helpers.run_live(command, stderr=stats_fh):
                    try:
                        j = json.loads(line)
                    except json.decoder.JSONDecodeError:
                        self.debug(f"Failed to decode line: {line}")
                        continue

                    decoder_name = j.get("DecoderName", "")

                    detector_name = j.get("DetectorName", "")

                    raw_result = j.get("Raw", "")

                    rawv2_result = j.get("RawV2", "")

                    verified = j.get("Verified", False)

                    source_metadata = j.get("SourceMetadata", {})

                    yield (decoder_name, detector_name, raw_result, rawv2_result, verified, source_metadata)
        finally:
            stats_file.unlink(missing_ok=True)

    def log_trufflehog_status(self, path, line):
        try:
            line = json.loads(line)
        except Exception:
            self.info(str(line))
            return
        message = line.get("msg", "")
        ts = line.get("ts", "")
        status = f"Message: {message} | Timestamp: {ts}"
        self.verbose(f"Current scan target: {path}")
        self.verbose(status)
